
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "constants.h"
#include "config.h"

char* F_ACTION_RESPONSE;
char* F_DICTIONARY;
char* F_TEMP;
char* F_TRUST_FILE;
char* F_PREFERENCES;
char* F_IGNORE_FILE;

/* configuration files */

#define FNAME_ACTION_RESPONSE  "profile"         /* Actions file */
#define FNAME_DICTIONARY       "dictionary"      /* Dictionary file */
#define FNAME_TEMP             "temp"            /* Temp file */
#define FNAME_PREFERENCES      "preferences.dat" /* stored preferences file */
#define FNAME_IGNORE_FILE      "ignorefile"      /* Ignored Users list */
#define FNAME_TRUST_FILE       "trustfile"       /* Trusted Users list */

#ifdef OS_LINUX
#define FNAME_SUBDIR ".yacca"
#else
#define FNAME_SUBDIR "data"
#endif

static char *
fname_to_full_path(const char *fname)
{
  char *buffer;
  char *home;
  size_t buffer_len;
  char *result;
  int temp;

#ifdef OS_LINUX
  home = getenv("HOME");
#else
  home = ".";
#endif

  if (home == NULL)
    home = ".";

  buffer_len = strlen(home) 
    + 1
    + strlen(FNAME_SUBDIR) 
    + 1
    + strlen(fname)
    + 1;

  buffer = calloc(buffer_len, 1);
  if (buffer == NULL)
    goto error_malloc;

  snprintf(buffer, buffer_len, "%s/%s/%s", home, FNAME_SUBDIR, fname);

  result = strdup(buffer);
  if (result == NULL)
    goto error_strdup;

  free(buffer);
  return result;

 error_strdup:
  temp = errno; free(buffer); errno = temp;
 error_malloc:
  return NULL;
}


int
constants_init()
{
  int temp;

  F_ACTION_RESPONSE = fname_to_full_path(FNAME_ACTION_RESPONSE);
  if (F_ACTION_RESPONSE == NULL) 
    goto error_action;
  F_DICTIONARY = fname_to_full_path(FNAME_DICTIONARY);
  if (F_DICTIONARY == NULL) 
    goto error_dictionary;
  F_TEMP = fname_to_full_path(FNAME_TEMP);
  if (F_TEMP == NULL) 
    goto error_temp;
  F_TRUST_FILE = fname_to_full_path(FNAME_TRUST_FILE);
  if (F_TRUST_FILE == NULL) 
    goto error_trust;
  F_PREFERENCES = fname_to_full_path(FNAME_PREFERENCES);
  if (F_PREFERENCES == NULL) 
    goto error_preferences;
  F_IGNORE_FILE = fname_to_full_path(FNAME_IGNORE_FILE);
  if (F_IGNORE_FILE == NULL) 
    goto error_ignore;

  return 0;

 error_ignore:
  temp = errno; free(F_PREFERENCES); errno = temp;
 error_preferences:
  temp = errno; free(F_TRUST_FILE); errno = temp;
 error_trust:
  temp = errno; free(F_TEMP); errno = temp;
 error_temp:
  temp = errno; free(F_DICTIONARY); errno = temp;
 error_dictionary:
  temp = errno; free(F_ACTION_RESPONSE); errno = temp;
 error_action:
  return 1;
}

void
constants_done()
{
  free(F_ACTION_RESPONSE);
  free(F_DICTIONARY);
  free(F_TEMP);
  free(F_TRUST_FILE);
  free(F_PREFERENCES);
  free(F_IGNORE_FILE);
}


/* Local Variables: */
/* tab-width: 3 */
/* End: */
