
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>

#include "listfiles.h"

#define BUFFER_MAX 2048

static gint 
key_compare_function(gconstpointer a, gconstpointer b, gpointer data)
{
  return strcmp(a, b);
}

static gint 
key_search_function(gconstpointer a, gconstpointer b)
{
  return strcmp(a, b);
}

static void
key_destroy_function(gpointer data)
{
  g_string_free(data, TRUE);
}

static gboolean
key_traversal_function(gpointer key, gpointer value, gpointer data)
{
  fprintf((FILE *) data, "%s\n", ((GString *) key)->str);

  return FALSE;
}


GTree *
file_to_tree(FILE *stream)
{
  GTree *result;
  char buffer[BUFFER_MAX];
  size_t len;

  result = g_tree_new_full(key_compare_function, NULL, key_destroy_function, NULL);
  while (fgets(buffer, BUFFER_MAX, stream) != NULL)
    {
      len = strlen(buffer);
      if (len > 0 && buffer[len - 1] == '\n')
        buffer[len - 1] = '\0';
      g_tree_insert(result, g_string_new(buffer), NULL);
    }
  if (ferror(stream))
    goto error_ferror;
  
  return result;

 error_ferror:
  g_tree_destroy(result);
  return NULL;
}

int
tree_to_file(GTree *tree, FILE *stream)
{
  g_tree_foreach(tree, key_traversal_function, stream);

  return 0;
}

int
string_in_tree(GTree *tree, const char string)
{
  return g_tree_search(tree, key_search_function, NULL) == NULL ? 0 : 1;
}


/* Local Variables: */
/* tab-width: 3 */
/* End: */
