
#include <stdio.h>
#include <errno.h>
#include <glib.h>

#include "prefs.h"
#include "commands.h"
#include "constants.h"


/* TODO: merge SavePreferences and CreatePreferences */

/* defines whether or not the bot should interact (toggle) 0 = on; 1 = off */
gint32 botMutePersonality = 0;

/* defines whether or not the bot should accept define statements (toggle) 0 = on; 1 = off */
gint32 botMuteDefinition = 0;

/* turn debug on or off (toggle) 0 = off; 1 = on */
gint32 DEBUG_MODE = 1;

typedef struct
{
  gint32 mutePersonality, muteDefinition;
  gint32 debugMode;
} prefData;


int
ReadPreferences(void)
{
	/* Read in the preferences from the preference file (if it exists) */
	prefData userPrefs;
	FILE *cfPref;
	int temp;

	cfPref = fopen(F_PREFERENCES, "r");
	if(cfPref == NULL)
	  goto error_fopen;

	if (fread(&userPrefs.mutePersonality, sizeof(userPrefs.mutePersonality), 1, cfPref) != 1)
	  goto error_fread;
	if (fread(&userPrefs.muteDefinition, sizeof(userPrefs.muteDefinition), 1, cfPref) != 1)
	  goto error_fread;
	if (fread(&userPrefs.debugMode, sizeof(userPrefs.debugMode), 1, cfPref) != 1)
	  goto error_fread;

	botMutePersonality = g_ntohl(userPrefs.mutePersonality);
	botMuteDefinition = g_ntohl(userPrefs.muteDefinition);
	DEBUG_MODE = g_ntohl(userPrefs.debugMode);

	fclose(cfPref);
	return 0;

 error_fread:
	temp = errno;
	fclose(cfPref);
	errno = temp;
 error_fopen:
	return 1;
}

int
SavePreferences(void)
{
	/* Save the current preferences into the preferences file (if it exists) */
	prefData userPrefs;
	FILE *cfPref;
	int temp;

	cfPref = fopen(F_PREFERENCES, "r+");
	if (cfPref == NULL)
	  goto error_fopen;

	userPrefs.mutePersonality = g_htonl(botMutePersonality);
	userPrefs.muteDefinition = g_htonl(botMuteDefinition);
	userPrefs.debugMode = g_htonl(DEBUG_MODE);

	if (fwrite(&userPrefs.mutePersonality, sizeof(userPrefs.mutePersonality), 1, cfPref) != 1)
	  goto error_fwrite;
	if (fwrite(&userPrefs.muteDefinition, sizeof(userPrefs.muteDefinition), 1, cfPref) != 1)
	  goto error_fwrite;
	if (fwrite(&userPrefs.debugMode, sizeof(userPrefs.debugMode), 1, cfPref) != 1)
	  goto error_fwrite;

	fclose(cfPref);
	return 0;

 error_fwrite:
	temp = errno;
	fclose(cfPref);
	errno = temp;
 error_fopen:
	return 1;
}

int CreatePreferences(void)
{
	/* Save the current preferences into the preferences file (if it exists) */
	prefData userPrefs;
	FILE *cfPref;
	int temp;

	cfPref = fopen(F_PREFERENCES, "w");
	if(cfPref == NULL)
	  goto error_fopen;

	userPrefs.mutePersonality = g_htonl(botMutePersonality);
	userPrefs.muteDefinition = g_htonl(botMuteDefinition);
	userPrefs.debugMode = g_htonl(DEBUG_MODE);

	if (fwrite(&userPrefs.mutePersonality, sizeof(userPrefs.mutePersonality), 1, cfPref) != 1)
	  goto error_fwrite;
	if (fwrite(&userPrefs.muteDefinition, sizeof(userPrefs.muteDefinition), 1, cfPref) != 1)
	  goto error_fwrite;
	if (fwrite(&userPrefs.debugMode, sizeof(userPrefs.debugMode), 1, cfPref) != 1)
	  goto error_fwrite;

	fclose(cfPref);
	return 0;

 error_fwrite:
	temp = errno;
	fclose(cfPref);
	errno = temp;
 error_fopen:
	return 1;
}


/* Local Variables: */
/* tab-width: 3 */
/* End: */
