/**********************************************************************
													ybot.c
This file is responsible for the main execution and loop of yacca.
Author: Joe Estock
Some functions by Justin Todd (webmaster@diversedev.com) (thanks Justin!)
Coded On: 03-09-2003 10.21pm
**********************************************************************/

/* Linux/Unix specific includes */
#ifdef OS_LINUX
	#include <unistd.h>
	#include <netdb.h>
	#include <sys/types.h>
	#include <netinet/in.h>
	#include <sys/socket.h>
#endif

/* Windows specific includes */
#ifdef OS_WINDOWS
	#include <winsock.h>
#endif

/* Standard includes */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "config.h"
#include "ycht_pkt.h"
#include "ybot.h"
#include "connection.h"
#include "commands.h"
#include "diagnostics.h"
#include "helpers.h"
#include "stats_basic.h"
#include "log_mysql.h"

/* defines whether or not the bot should interact (toggle) 0 = on; 1 = off */
static int botMutePersonality 	= 0;

/* defines whether or not the bot should accept define statements (toggle) 0 = on; 1 = off */
static int botMuteDefinition		= 0;

/* turn debug on or off (toggle) 0 = off; 1 = on */
static int DEBUG_MODE 			= 1;

/* maximum length of a define keyword (default is 12) */
static int botMaxDefineLen		= 20;

/* the bot's username (taken from the first parameter of the command line argument) */
static char BOT_USERNAME[MAX_UNAME_LEN];

struct prefData
{
	int mutePersonality;
	int muteDefinition;
	int debugMode;
};

void usage(void)
{
	printf("YaCCa Chat Bot Version 0.2.4\n");
	printf("Usage: yacca [username] [password] [room to join]\n");
	printf("[username] = yahoo id for yacca\n");
	printf("[password] = yahoo password\n");
	printf("[room to join] = the room you want to join\n");
}

int ReadPreferences(void)
{
	/* Read in the preferences from the preference file (if it exists) */
	struct prefData userPrefs;
	FILE * cfPref;
	cfPref = fopen(F_PREFERENCES, "r");
	if(cfPref == NULL)
	{
		/* unable to open preferences -- calling function will create a default one (je) */
		return 1;
	}
	else
	{
		fread(&userPrefs, sizeof(userPrefs), 1, cfPref);
	}

	if(&userPrefs.mutePersonality != NULL) { botMutePersonality = userPrefs.mutePersonality; }
	if(&userPrefs.muteDefinition != NULL) { botMuteDefinition = userPrefs.muteDefinition; }
	if(&userPrefs.debugMode != NULL) { DEBUG_MODE = userPrefs.debugMode; }

	fclose(cfPref);
	return 0;
}

int SavePreferences(void)
{
	/* Save the current preferences into the preferences file (if it exists) */
	struct prefData userPrefs;
	FILE * cfPref;
	cfPref = fopen(F_PREFERENCES, "r+");

	if(cfPref == NULL)
	{
		/* unable to save the preferences file -- damn (je) */
		return 1;
	}
	else
	{
		userPrefs.mutePersonality = botMutePersonality;
		userPrefs.muteDefinition = botMuteDefinition;
		userPrefs.debugMode = DEBUG_MODE;

		fwrite(&userPrefs, sizeof(userPrefs), 1, cfPref);
	}

	fclose(cfPref);
	return 0;
}

/* added this to create the preferences file if none exists (je) */
int CreatePreferences(void)
{
	/* Save the current preferences into the preferences file (if it exists) */
	struct prefData userPrefs;
	FILE * cfPref;
	cfPref = fopen(F_PREFERENCES, "w");

	if(cfPref == NULL)
	{
		/* unable to save the preferences file -- damn (je) */
		return 1;
	}
	else
	{
		userPrefs.mutePersonality = botMutePersonality;
		userPrefs.muteDefinition = botMuteDefinition;
		userPrefs.debugMode = DEBUG_MODE;

		fwrite(&userPrefs, sizeof(userPrefs), 1, cfPref);
	}

	fclose(cfPref);
	return 0;
}

int main(int argc,char *argv[])
{
	char cookie[256],rbuf[1024],msg[1024],send_msg[512];
	char *room,superuser[MAX_UNAME_LEN],sender_name[MAX_UNAME_LEN];
	int S;								/* our chat session connection variable (je) */
	int i = 0;							/* used exclusively for loops (je) */
	int iRetResult = 0;		/* universal function returns (je) */
	fd_set fdread;
	time_t start_t,ping_t;
	Tstat stats;
	struct ItemNode *itemlist=NULL;

	/* we need to inform the user of the loading process (je) */
	printf("Loading saved preferences...");
	fflush(stdout);

	iRetResult = ReadPreferences();		/* 0 = success; 1 = error */

	if(iRetResult == 1)
	{
		if(DEBUG_MODE == 1)
			printf("Failed\nCreating new preferences file...");
		else
			printf("Done\n");
		
			fflush(stdout);
		
		/* attempt to create a new preferences file since none exists (je) */
		iRetResult = CreatePreferences();
		if(iRetResult == 1)
		{
			/* unable to create the preferences file for some reason so use the hard-coded preferences (je) */
			if(DEBUG_MODE == 1)
				printf("Failed\nUsing default preferences.\n");
		}
		else
			printf("Done\n");
		
		fflush(stdout);
	}
	else
		printf("Done\n");
	
		fflush(stdout);

	#ifdef OS_WINDOWS
		WSADATA wsaData;
	#endif

	/* check the commandline arguments */
	if(argc!=4)
	{
		usage();
		exit(1);
	}

	#ifdef OS_WINDOWS
		if(WSAStartup(MAKEWORD(1, 1), &wsaData) != 0)
		{
			fprintf(stderr, "WSAStartup failed.\n");
			exit(1);
		}
	#endif

	/* copy room argument to a nicer named var... we use it alot */
	room=argv[3];
	
	/* copy the bot's username */
	strcpy(BOT_USERNAME, argv[1]);

	/* see who will own the bot for this session */
	printf("Enter your Yahoo! username: ");
	fflush(stdout);
	scanf("%s",superuser);

	/* removed conditional expression because we need to inform the user of the process (je) */
	printf("Waiting for cookie...");
	fflush(stdout);

	/* Get the cookie */
	GetCookie(cookie,argv[1],argv[2]);
	
	/* removed conditional expression because we need to inform the user of the process (je) */
	printf("Done\nConnecting to chat server...");
	fflush(stdout);

	/* connect to the chat server */
	if((S=Connect(CHATSERVER,CHATPORT))==-1)
	{
		fprintf(stderr,"Connection to chat server failed!");
		exit(1);
	}

	/* removed conditional expression because we need to inform the user of the process (je) */
	printf("Connected\nLogging into chat server...");
	fflush(stdout);

	ItemAdd(&itemlist,argv[1]);
	ItemAdd(&itemlist,cookie);
	SendYCHTPkt(S,YCHT_LOGIN,&itemlist);
	GetPacket(S,rbuf);
	
	/* removed conditional expression because we need to inform the user of the process (je) */
	printf("Done\nSending join packet...");
	fflush(stdout);

	/* make and send a join packet.. wait for a response */
	ItemAdd(&itemlist,room);
	SendYCHTPkt(S,YCHT_JOIN,&itemlist);
	
	GetPacket(S,rbuf);

	/* removed conditional expression because we need to inform the user of the process (je) */
	printf("Done\n");
	fflush(stdout);

	/* display annoying about message */
	ItemAdd(&itemlist,room);
	strcpy(send_msg,"yacca is in the hizzouse! ;) ");

	/* generic entry message...perhaps we should make it configurable? (je) */
	#ifdef OS_WINDOWS
		strncat(send_msg,"Running on Windows, baby!",25);
	#endif
	#ifdef OS_LINUX
		strncat(send_msg,"Runnin on Linux, baby!",22);
	#endif

	ItemAdd(&itemlist,send_msg);
	SendYCHTPkt(S,YCHT_TALK,&itemlist);

	/* Main program loop */

	time(&start_t);				/* record the start time */
	time(&ping_t);				/* ping timeout */
	ResetStats(&stats);		/* reset all the stats to zero */

	for(;;)
	{
		FD_ZERO(&fdread);
		FD_SET(S,&fdread);
		select(S+1,&fdread,NULL,NULL,NULL);

		/* check to see if a ping should be sent (ping time out can be altered as seen neccassary */
		if(difftime((time_t)time(NULL),ping_t) >= PING_TIMEOUT)
		{
			SendYCHTPkt(S,YCHT_PING,NULL);
			time(&ping_t);
		}

		/* A packet has arrived! */
		if(FD_ISSET(S,&fdread))
		{
			GetPacket(S,rbuf);

			if(DEBUG_MODE == 1)
			{
				DumpYCHTPkt(rbuf);			/* dump the packet to stdout for debugging purposes */
				fflush(stdout);						/* speed up the dump process (je) */
			}

			/* do packet statistics */
			DoRoomStats(rbuf, &stats);

			/* do packet logging NOTE: this will no longer be needed (or supported) since we will use a flat-file (je)*/
			#ifdef MYSQL_SUPPORT
				logMySQL(rbuf);
			#endif

			/* received a talk packet */
			if(pktGetType(rbuf)==PKT_TALK)
			{
				/* get the sender and message from the packet */
				pktGetDataItem(rbuf,sender_name,1);
				pktGetDataItem(rbuf,msg,2);

				/* strip out all the tags so we're left with plain text */
				strip_tags(msg,"<",">");
				strip_tags(msg,"\x1B","m");

				i = 0;		/* reset to 0 to clear out previous usage if any (je) */
				while(i < strlen(msg))
				{
					if(isupper(msg[i]))
						msg[i] = tolower(msg[i]);

						i++;
				}

				/* see if user is requesting a definition */
				if(!strncmp(msg, "yacca define ",13))
				{
					/* ignore the bot's username, otherwise he will talk to himself */
					if(strcmp(sender_name, BOT_USERNAME))
					{
						/* ensure that active user is not on ignore list */
						if(IsIgnored(sender_name) == 0)
						{
							/* finally, ensure that we are not muted */
							if(botMuteDefinition != 1)
							{
								/* ensure that the word is not more than botMaxDefineLen */
								if(strlen(msg+13) > botMaxDefineLen)
								{
									strcpy(send_msg, "The word you entered is too long, ");
									strncat(send_msg, sender_name, strlen(sender_name));
								}
								else
								{
									iRetResult = DefineResponse(send_msg,msg+13,sender_name);

									if(iRetResult != 1)
									{
										/* No definition was found */
										strcpy(send_msg,"I'm sorry ");
										strncat(send_msg, sender_name, strlen(sender_name));
										strncat(send_msg, ", however I could not find the definition for '",47);
										strncat(send_msg,msg+13, strlen(msg+13));
										strncat(send_msg, "'.", 2);
									}
								} /* end if */

								ItemAdd(&itemlist,room);
								ItemAdd(&itemlist,send_msg);
								SendYCHTPkt(S,YCHT_TALK,&itemlist);
							} /* end if */
						} /* end if */
					} /* end if */
				} /* end if */
				else
				{
					/* ACTIONS / RESPONSES (taken from file: data/personality) */
					if(ActionResponse(send_msg,msg,sender_name,room))
					{
						/* ignore the bot's username, otherwise he will talk to himself */
						if(strcmp(sender_name, BOT_USERNAME))
						{
							/* ensure that active user is not on ignore list */
							if(IsIgnored(sender_name) == 0)
							{
								/* make sure we are allowed to chat */
								if(botMutePersonality != 1)
								{
									ItemAdd(&itemlist,room);
									ItemAdd(&itemlist,send_msg);
									SendYCHTPkt(S,YCHT_TALK,&itemlist);
								}
							} /* end if */
						} /* end if */
					} /* end if */
				} /* end if */

				i = 0;
				while(i < strlen(msg))
				{
					msg[i] = tolower(msg[i]);
					i++;
				}

				/* Parse for commands */
				if(IsTrusted(sender_name) || !strcmp(sender_name,superuser))
				{
					if(!strcmp(msg,"room stats"))														/* display the room statistics */
						Stat2Str(send_msg,&stats);
					else if(!strcmp(msg,"server time"))											/* display the server time */
						time2str(send_msg);
					else if(!strcmp(msg,"yacca uptime"))										/* display how long yacca has been running */
						uptime2str(send_msg,start_t);
					else if(!strncmp(msg,"yacca add user ",15))						/* add a user to the trustlist */
					{
						UserAdd(msg+15);
						strcpy(send_msg,"user added");
					}
					else if(!strncmp(msg,"yacca delete user ",18))					/* delete a user from the trustlist */
					{
						UserDel(msg+18);
						strcpy(send_msg,"user removed");
					}
					else if(!strncmp(msg,"yacca ignore user ",18))					/* add a user to the ignore file */
					{
						/* see if user has previously been ignored */
						if(IsIgnored(msg+18) == 0)
						{
							IgnoreUser(msg+18);
							strcpy(send_msg, msg+18);
							strncat(send_msg, " has been ignored",17);
						}
						else
						{
							/* user is already on ignore so no use adding them twice */
							strcpy(send_msg, msg+18);
							strncat(send_msg, " is already on ignore",21);
						}
					}
					else if(!strncmp(msg,"yacca unignore user ",20))				/* remove a user from the ignore file */
					{
						/* see if user has previously been ignored */
						if(IsIgnored(msg+20) == 0)
						{
							UnignoreUser(msg+20);
							strcpy(send_msg, msg+20);
							strncat(send_msg, " has been unignored",19);
						}
						else
						{
							/* you can't remove something that's not there */
							strcpy(send_msg, msg+20);
							strncat(send_msg, " is not currently ignored",25);
						}
					}
					else if(!strcmp(msg,"yacca trusted users"))							/* list the users in the trusted list */
						file2str(send_msg,F_TRUSTLIST);
					else if(!strcmp(msg,"yacca shutdown"))								/* close down yacca */
					{
						strcpy(send_msg, "Goodbye everyone");
						ItemAdd(&itemlist,room);
						ItemAdd(&itemlist,send_msg);
						SendYCHTPkt(S,YCHT_TALK,&itemlist);
						Disconnect(S);
						SavePreferences();
						exit(0);
					}
					else if(!strcmp(msg,"yacca owner"))										/* display the current owner */
					{
						strcpy(send_msg, superuser);
						strncat(send_msg, " is the current superuser", 13);
					}
					else if(!strcmp(msg,"yacca version"))									/* display the version */
					{
						strcpy(send_msg, "yacca - version 0.2.4");
					}
					else if(!strncmp(msg,"yacca join ",11))									/* join a chat room */
					{
						if(!strcmp(room, msg+11))
						{
							strcpy(send_msg, "I'm already in ");
							strncat(send_msg, room, strlen(room));
						}
						else
						{
							strcpy(send_msg, "Goodbye everyone");
							ItemAdd(&itemlist,room);
							ItemAdd(&itemlist,send_msg);
							SendYCHTPkt(S,YCHT_TALK,&itemlist);

							strcpy(room,msg+11);
							ItemAdd(&itemlist,room);
							SendYCHTPkt(S,YCHT_JOIN,&itemlist);
							GetPacket(S,rbuf);
							sprintf(send_msg,"I just came from %s",room);
						}
					}
					else if(!strncmp(msg,"yacca set superuser = ",22))			/* change the superuser */
					{
						sprintf(send_msg,"%s no longer the superuser",superuser);
						strcpy(superuser,msg+22);
					}
					else if(!strncmp(msg, "yacca debug on",14))						/* turn on terminal output */
					{
						DEBUG_MODE = 1;
						strcpy(send_msg, "debug trace ON");
					}
					else if(!strncmp(msg, "yacca debug off",15))						/* turn off terminal output */
					{
						DEBUG_MODE = 0;
						strcpy(send_msg, "debug trace OFF");
					}
					else if(!strncmp(msg, "yacca mute personality",22))			/* stop or start responding to user events */
					{
						if(botMutePersonality == 0)
						{
							botMutePersonality = 1;
							strcpy(send_msg, "Personality matrix OFF");
						}
						else
						{
							botMutePersonality = 0;
							strcpy(send_msg, "Personality matrix ON");
						}
					}
					else if(!strncmp(msg, "yacca mute definition",21))				/* stop or start responding to dictionary requests */
					{
						if(botMuteDefinition == 0)
						{
							botMuteDefinition = 1;
							strcpy(send_msg, "Definition matrix OFF");
						}
						else
						{
							botMuteDefinition = 0;
							strcpy(send_msg, "Definition matrix ON");
						}
					}
					else if(!strncmp(msg, "yacca status",12))								/* display status of dictionary and user requests */
					{
						if(botMuteDefinition == 1)
						{
							strcpy(send_msg, "Definition matrix OFF, ");
						}
						else
						{
							strcpy(send_msg, "Definition matrix ON, ");
						}

						if(botMutePersonality == 1)
						{
							strncat(send_msg, "Personality matrix OFF", 22);
						}
						else
						{
							strncat(send_msg, "Personality matrix ON", 21);
						}
					}
					else continue;  /* no command was entered so continue loop from the beginning */

					/* send the command status */
					ItemAdd(&itemlist,room);
					ItemAdd(&itemlist,send_msg);
					SendYCHTPkt(S,YCHT_TALK,&itemlist);
				}
			}
		}
	}
	
	Disconnect(S);
	exit(0);
}
